/*
 * _74HC595.c
 */ 

#include <avr/io.h>
#include "_74HC595.h"

void init_74HC595()
{
	HC74_595_DDR |= ((1<<RCK) | (1<<SCK)  | (1<<SIN));
	shift_all_assume();
}

// Alle Daten aus HC595_Data schieben, anschlieend Daten bernehmen
void shift_all_assume()
{
	for (int i = anzahl_74HC595 -1;  i > -1; i--)
	shift_Data_74HC595(HC595_Data[i]);
	assume_Data_74HC595();
}

// Softwareansteuerung Schieberegister
// Daten werden in das Schieberegister geschoben
void shift_Data_74HC595(uint8_t pins)
{
	uint8_t temp = 128;
	for (uint8_t i = 8; i > 0; i--)
	{
		// ist Bit in Ausgabe gesetzt ?
		if ( (pins & temp ) )
		{
			HC74_595_PORT |= (1<<SIN);
		}
		else
		{
			HC74_595_PORT &=~ (1<<SIN);
		}
		// Clock Puls
		HC74_595_PORT |= (1<<SCK);
		HC74_595_PORT &=~ (1<<SCK);
		temp >>= 1;
	}
}

// Daten im Schieberegister bernehmen
void assume_Data_74HC595()
{
	// RCK 010 Signal ausgeben
	HC74_595_PORT |= (1<<RCK);
	HC74_595_PORT &=~ (1<<RCK);
	// Pin wird auf 0 gesetzt, damit die Led am Sparrow nicht leuchtet
	HC74_595_PORT &=~ (1<<SIN);
}

// einzelnen Pin setzen oder lschen,
// s_Register_Nummer	- Nummer des Schieberegisters, von 0 bis x
// s_Pin				- Nummer des zu ndernden Pin, von 0 bis 7
void set_Pin_74HC595(uint8_t s_Register_Nummer, uint8_t s_Pin)
{
	HC595_Data[s_Register_Nummer] |= (1 << s_Pin);
	shift_all_assume();
}
void clear_Pin_74HC595(uint8_t s_Register_Nummer, uint8_t s_Pin)
{
	HC595_Data[s_Register_Nummer] &=~ (1 << s_Pin);
	shift_all_assume();
}

// kompletten "Port" setzen oder lschen,
// s_Register_Nummer	- Nummer des Schieberegisters, von 0 bis x
// s_Pin				- Pinmuster ( s_Pin 5 -> Pin 0,2 aktiv s_Pin 42 -> Pin 1,3,5 aktiv ...)
void set_Pins_74HC595(uint8_t s_Register_Nummer, uint8_t s_Pin)
{
	HC595_Data[s_Register_Nummer] = s_Pin;
	shift_all_assume();
}
void clear_Pins_74HC595(uint8_t s_Register_Nummer, uint8_t s_Pin)
{
	HC595_Data[s_Register_Nummer] = s_Pin;
	shift_all_assume();
}